"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.logger = exports.fromIframe = exports.fromDocument = exports.headStyleTagsId = exports.highlightSelectedClassName = exports.highlightClassName = exports.setDebug = exports.HIGHLIGHT_IGNORE_ATTR = exports.IGNORE_ELEMENT_REACT_ATTR = exports.IGNORE_ELEMENT_ATTR = void 0;
const invariant_1 = __importDefault(require("invariant"));
const addEvent_1 = __importDefault(require("@thinkeloquent/rd2020-dom/src/element/addEvent"));
const addClass_1 = __importDefault(require("@thinkeloquent/rd2020-dom/src/element/addClass"));
const removeClass_1 = __importDefault(require("@thinkeloquent/rd2020-dom/src/element/removeClass"));
const addAttribute_1 = __importDefault(require("@thinkeloquent/rd2020-dom/src/element/addAttribute"));
const getTargetFromEvent_1 = __importDefault(require("@thinkeloquent/rd2020-dom/src/element/getTargetFromEvent"));
const findOne_1 = require("@thinkeloquent/rd2020-dom/src/selector/findOne");
const find_1 = require("@thinkeloquent/rd2020-dom/src/selector/find");
const getTagName_1 = __importDefault(require("@thinkeloquent/rd2020-dom/src/element/getTagName"));
const getNodeTypeFromElement_1 = __importStar(require("@thinkeloquent/rd2020-dom/src/element/getNodeTypeFromElement"));
const index_less_1 = __importDefault(require("./index.less"));
exports.IGNORE_ELEMENT_ATTR = 'hightlight-ignore-element';
exports.IGNORE_ELEMENT_REACT_ATTR = { 'hightlight-ignore-element': true };
exports.HIGHLIGHT_IGNORE_ATTR = 'hightlight-ignore-element';
let DEBUG = false;
exports.setDebug = (boolean) => DEBUG = Boolean(boolean);
exports.highlightClassName = `root-${process.env.BUILD_ID}`;
exports.highlightSelectedClassName = `selected-${process.env.BUILD_ID}`;
exports.headStyleTagsId = `style-${process.env.BUILD_ID}`;
exports.fromDocument = (document) => ({
    body: document.body,
    head: document.head,
    document,
});
exports.fromIframe = (document) => ({
    body: document.body,
    head: document.head,
    document,
});
exports.logger = (...args) => {
    if (!DEBUG)
        return;
    console.log('ELEMENT_HIGHLIGHT', ...args);
};
class ElementHighlight {
    constructor(element) {
        const isDocument = getNodeTypeFromElement_1.default(element).nodeType === getNodeTypeFromElement_1.DOCUMENT_NODE;
        const { body, head, document, } = isDocument ? exports.fromDocument(element) : exports.fromIframe(element);
        this.dom = {
            body,
            head,
            document,
        };
        this.previousHoveredElement = null;
        this.previousSelectedElement = null;
        this.disposeOf = [];
        this.classes = {
            ignoreElement: index_less_1.default.locals.highlight,
            highlight: index_less_1.default.locals.highlight,
            highlightSelected: index_less_1.default.locals.highlightSelected,
        };
        this.insertStyleSheet();
    }
    setIgnoreAttribute(element) {
        if (element) {
            element.setAttribute(exports.HIGHLIGHT_IGNORE_ATTR, "true");
        }
    }
    useStyleSheet() {
        if (index_less_1.default.use)
            index_less_1.default.use();
    }
    unuseStyleSheet() {
        if (index_less_1.default.unuse)
            index_less_1.default.unuse();
    }
    insertStyleSheet() {
        this.stylesheet = document.createElement('style');
        this.stylesheet.id = exports.headStyleTagsId;
        this.stylesheet.innerHTML = index_less_1.default.toString();
        this.dom.head.insertAdjacentHTML('beforeend', this.stylesheet.outerHTML);
    }
    removeClassName(className) {
        let currentlySelected = find_1.findWithDocument(this.dom.document, `.${className}`);
        if (currentlySelected && currentlySelected.length) {
            removeClass_1.default(currentlySelected, className);
        }
        return currentlySelected;
    }
    getSelected() {
        return findOne_1.findOneWithDocument(this.dom.document, this.classes.highlightSelected);
    }
    ignoreElement(query) {
        const elements = find_1.findWithDocument(this.dom.document, query);
        if (!elements) {
            exports.logger(`ignoreElement element not found: ${query}`);
            return;
        }
        addAttribute_1.default(elements, exports.IGNORE_ELEMENT_ATTR, "true");
    }
    stopPropagation(query) {
        const element = findOne_1.findOneWithDocument(this.dom.document, query);
        if (!element) {
            exports.logger(`stopPropagation element not found: ${query}`);
            return;
        }
        addEvent_1.default(element, 'click', (event) => {
            exports.logger(`ignore element ${query}`);
            event.preventDefault();
            event.stopPropagation();
        });
    }
    disposal() {
        for (let index in this.disposeOf) {
            this.disposeOf[index]();
            delete this.disposeOf[index];
        }
    }
    cleanup() {
        this.removeHighlight();
        this.disposal();
    }
    removeHighlight() {
        this.previousHoveredElement = null;
        this.previousSelectedElement = null;
        this.removeClassName(this.classes.highlight);
        this.removeClassName(this.classes.highlightSelected);
    }
    mouseover({ editableTags, onHoverIgnore, }) {
        this.disposeOf.push(addEvent_1.default(this.dom.body, 'mouseover', (event) => {
            const target = getTargetFromEvent_1.default(event);
            const tagName = getTagName_1.default(target);
            const ignore = target.getAttribute(exports.IGNORE_ELEMENT_ATTR);
            if (ignore === "true") {
                exports.logger(`IGNORE_ELEMENT_ATTR element mouseover: ${tagName}`);
                if (typeof onHoverIgnore === 'function')
                    onHoverIgnore(target, this.previousSelectedElement);
                return;
            }
            if (editableTags.indexOf(tagName) === -1) {
                exports.logger(`ignore element mouseover: ${tagName}`);
                if (typeof onHoverIgnore === 'function')
                    onHoverIgnore(target, this.previousSelectedElement);
                return;
            }
            addClass_1.default(target, this.classes.highlight);
            this.previousHoveredElement = target;
        }));
    }
    mouseout() {
        this.disposeOf.push(addEvent_1.default(this.dom.body, 'mouseout', (event) => {
            const element = event ? event.target : this.previousHoveredElement;
            if (element)
                removeClass_1.default(element, this.classes.highlight);
        }));
    }
    click(options) {
        const { onClickIgnore, editableTags, onReSelect, } = options;
        this.disposeOf.push(addEvent_1.default(this.dom.body, 'click', (event) => {
            const target = getTargetFromEvent_1.default(event);
            const tagName = getTagName_1.default(target);
            const ignore = target.getAttribute(exports.IGNORE_ELEMENT_ATTR);
            if (ignore === "true") {
                exports.logger(`IGNORE_ELEMENT_ATTR element click: ${tagName}`);
                if (typeof onClickIgnore === 'function')
                    onClickIgnore(target, this.previousSelectedElement);
                this.returnValue = true;
                return true;
            }
            event.preventDefault();
            event.stopPropagation();
            if (this.previousSelectedElement === target) {
                if (typeof onReSelect === 'function')
                    onReSelect(this.previousSelectedElement);
                exports.logger(`reselect ignore click: ${tagName}`);
                return;
            }
            if (editableTags.indexOf(tagName) === -1) {
                exports.logger(`ignore element click: ${tagName}`);
                if (typeof onClickIgnore === 'function')
                    onClickIgnore(target, this.previousSelectedElement);
                return;
            }
            this.selectElement(target, options);
        }));
    }
    selectElement(target, { onSelect, onRemove, }) {
        if (this.previousSelectedElement) {
            if (typeof onRemove === 'function')
                onRemove(target, this.previousSelectedElement);
            removeClass_1.default(this.previousSelectedElement, this.classes.highlightSelected);
        }
        addClass_1.default(target, this.classes.highlightSelected);
        if (typeof onSelect === 'function')
            onSelect(target, this.previousSelectedElement);
        this.previousSelectedElement = target;
    }
    initialize(argOptions = {}) {
        const options = Object.assign({}, argOptions);
        invariant_1.default(Array.isArray(options.editableTags), 'editableTags array is missing');
        this.mouseover(options);
        this.mouseout(options);
        this.click(options);
    }
    getData() {
        return {};
    }
}
exports.default = ElementHighlight;
//# sourceMappingURL=index.js.map